/* arch/arm/mach-s3c6400/include/mach/regs-fb.h
 *
 * Copyright 2008 Openmoko, Inc.
 * Copyright 2008 Simtec Electronics
 *      http://armlinux.simtec.co.uk/
 *      Ben Dooks <ben@simtec.co.uk>
 *
 * S3C64XX - new-style framebuffer register definitions
 *
 * This is the register set for the new style framebuffer interface
 * found from the S3C2443 onwards and specifically the S3C64XX series
 * S3C6400 and S3C6410.
 *
 * The file contains the cpu specific items which change between whichever
 * architecture is selected. See <plat/regs-fb.h> for the core definitions
 * that are the same.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
*/

/* include the core definitions here, in case we really do need to
 * override them at a later date.
*/

#include <plat/regs-fb.h>

#define S3C_FB_MAX_WIN (2)  /* number of hardware windows available. */

/* Video timing controls */
#define VIDTCON0				(0x08)
#define VIDTCON1				(0x0C)
#define VIDTCON2				(0x10)

/* Window position controls */

#define WINCON(_win)				(0x14 + ((_win) * 4))

/* OSD1 do not have register C */

#define VIDOSD_A(_win)				(0x28 + ((_win) * 12))
#define VIDOSD_B(_win)				(0x2C + ((_win) * 12))
#define VIDOSD_C(_win)				(0x30 + ((_win) * 12))

/* Video buffer addresses */

#define VIDW_BUF_START(_buff)			(0x64 + ((_buff) * 8))
#define VIDW_BUF_START1(_buff)			(0x68 + ((_buff) * 8))
#define VIDW_BUF_END(_buff)			(0x7C + ((_buff) * 8))
#define VIDW_BUF_END1(_buff)			(0x80 + ((_buff) * 8))
#define VIDW_BUF_SIZE(_buff)			(0x94 + ((_buff) * 8))

#define VIDINTCON0				(0xAC)

#define WxKEYCONy(_win, _con)			((0xB0 + ((_win) * 8)) + ((_con) * 4))

#define WINxMAP(_win)				(0xD0 + ((_win) * 4))

#define WPALCON					(0xE4)

#define DITHMODE				(0x138)

/* Palette registers */

#define WIN0_PAL(_entry)			(0x400 + ((_entry) * 4))
#define WIN1_PAL(_entry)			(0x800 + ((_entry) * 4))

/* system specific implementation code for palette sizes, and other
 * information that changes depending on which architecture is being
 * compiled.
*/

/* return true if window _win has OSD register C */
#define s3c_fb_has_osd_c(_win) (_win)
/* return true if window _win has OSD register D */
#define s3c_fb_has_osd_d(_win) (0)

static inline unsigned int s3c_fb_win_pal_size(unsigned int win)
{
	if (win < 2)
		return 256;

	BUG();	/* shouldn't get here */
}

static inline int s3c_fb_validate_win_bpp(unsigned int win, unsigned int bpp)
{
	if (bpp == 1 || bpp == 2 || bpp == 4 || bpp == 8 ||
	    bpp == 16 || bpp == 18 || bpp == 24)
		return 1;
	return 0;
}

static inline unsigned int s3c_fb_pal_reg(unsigned int window, int reg)
{
	switch (window) {
	case 0: return WIN0_PAL(reg);
	case 1: return WIN1_PAL(reg);
	}

	BUG();
}

static inline int s3c_fb_pal_is16(unsigned int window)
{
	return window > 1;
}

struct s3c_fb_palette {
	struct fb_bitfield	r;
	struct fb_bitfield	g;
	struct fb_bitfield	b;
	struct fb_bitfield	a;
};

static inline void s3c_fb_init_palette(unsigned int window,
				       struct s3c_fb_palette *palette)
{
	if (window < 2) {
		/* Windows 0/1 are 8/8/8 or A/8/8/8 */
		palette->r.offset = 16;
		palette->r.length = 8;
		palette->g.offset = 8;
		palette->g.length = 8;
		palette->b.offset = 0;
		palette->b.length = 8;
	} else {
		/* currently we assume RGB 5/6/5 */
		palette->r.offset = 11;
		palette->r.length = 5;
		palette->g.offset = 5;
		palette->g.length = 6;
		palette->b.offset = 0;
		palette->b.length = 5;
	}
}
