/* arch/arm/mach-s3c2410/include/mach/regs-s3c2416-clock.h
 *
 * Copyright (c) 2007 Simtec Electronics
 *	Ben Dooks <ben@simtec.co.uk>
 *	http://armlinux.simtec.co.uk/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * S3C2416 clock register definitions
*/

#ifndef __ASM_ARM_REGS_S3C2416_CLOCK
#define __ASM_ARM_REGS_S3C2416_CLOCK

#include <mach/regs-s3c2443-clock.h>

#define S3C2416_MPLLCON_MDIVSHIFT	14
#define S3C2416_MPLLCON_PDIVSHIFT	5
#define S3C2416_MPLLCON_SDIVSHIFT	0
#define S3C2416_MPLLCON_MDIVMASK	((1<<(1+(23-14)))-1)
#define S3C2416_MPLLCON_PDIVMASK	((1<<(1+(10-5)))-1)
#define S3C2416_MPLLCON_SDIVMASK	(7)

#define S3C2416_EPLLCON_MDIVSHIFT	16
#define S3C2416_EPLLCON_PDIVSHIFT	8
#define S3C2416_EPLLCON_SDIVSHIFT	0
#define S3C2416_EPLLCON_MDIVMASK	((1<<(1+(23-16)))-1)
#define S3C2416_EPLLCON_PDIVMASK	((1<<(1+(13-8)))-1)
#define S3C2416_EPLLCON_SDIVMASK	(7)

#define S3C2416_EPLLCON_K_KDIVSHIFT	0
#define S3C2416_EPLLCON_K_KDIVMASK	0xFFFF

#define S3C2416_EPLLCON_K		S3C2443_CLKREG(0x1C)
#define S3C2443_CLKDIV2			S3C2443_CLKREG(0x2C)

#define S3C2416_CLKSRC_HSSPI0_MPLLDIV	(1<<18)
#define S3C2416_CLKSRC_HSMMC1_EXTCLK	(1<<17)
#define S3C2416_CLKSRC_HSMMC0_EXTCLK	(1<<16)

#define S3C2416_CLKDIV0_ARMDIV_MASK     (7<<9)
#define S3C2416_CLKDIV0_ARMDIV_SHIFT    (9)
#define S3C2416_CLKDIV0_ARMDIV_1        (0<<9)
#define S3C2416_CLKDIV0_ARMDIV_2        (1<<9)
#define S3C2416_CLKDIV0_ARMDIV_3        (2<<9)
#define S3C2416_CLKDIV0_ARMDIV_4        (3<<9)
#define S3C2416_CLKDIV0_ARMDIV_6        (5<<9)
#define S3C2416_CLKDIV0_ARMDIV_8        (7<<9)

#if 0
#define S3C2416_CLKDIV2_SPIDIV_MPLL_MASK	(31<<4)
#define S3C2416_CLKDIV2_SPIDIV_MPLL_SHIFT	(0)
#endif

#define S3C2416_HCLKCON_G2D		(1<<20) 
#define S3C2416_HCLKCON_HSMMC0		(1<<15)

#define S3C2416_SCLKCON_HSMMC0CLK_EPLL  (1<<6)

#define S3C2416_SWRST_RESET		(0x533c2416)

#define S3C2416_RSTSTAT			S3C2443_CLKREG(0x68)
#define S3C2416_WKUPSTAT		S3C2443_CLKREG(0x6C)

#include <asm/div64.h>

static inline unsigned int
s3c2416_get_mpll(unsigned int pllval, unsigned int baseclk)
{
	unsigned int mdiv, pdiv, sdiv;
	uint64_t fvco;

	mdiv = pllval >> S3C2416_MPLLCON_MDIVSHIFT;
	pdiv = pllval >> S3C2416_MPLLCON_PDIVSHIFT;
	sdiv = pllval >> S3C2416_MPLLCON_SDIVSHIFT;

	mdiv &= S3C2416_MPLLCON_MDIVMASK;
	pdiv &= S3C2416_MPLLCON_PDIVMASK;
	sdiv &= S3C2416_MPLLCON_SDIVMASK;

	fvco = (uint64_t)baseclk * mdiv;
	do_div(fvco, pdiv << sdiv);

	return (unsigned int)fvco;
}

static inline unsigned int
s3c2416_get_epll(unsigned int pllval, unsigned int pllval_k, unsigned int baseclk)
{
	unsigned int mdiv, pdiv, sdiv, kdiv;
	uint64_t fvco;

	mdiv = pllval >> S3C2416_EPLLCON_MDIVSHIFT;
	pdiv = pllval >> S3C2416_EPLLCON_PDIVSHIFT;
	sdiv = pllval >> S3C2416_EPLLCON_SDIVSHIFT;
	kdiv = pllval_k >> S3C2416_EPLLCON_K_KDIVSHIFT;

	mdiv &= S3C2416_EPLLCON_MDIVMASK;
	pdiv &= S3C2416_EPLLCON_PDIVMASK;
	sdiv &= S3C2416_EPLLCON_SDIVMASK;
	kdiv &= S3C2416_EPLLCON_K_KDIVMASK;

	fvco = (uint64_t)baseclk * mdiv + (((uint64_t)baseclk * kdiv) >> 16);
	do_div(fvco, pdiv << sdiv);

	return (unsigned int)fvco;
}

#endif /*  __ASM_ARM_REGS_S3C2416_CLOCK */
