/* linux/arch/arm/mach-s3c2443/mach-snp2100.c
 *
 * Copyright (c) 2007 Simtec Electronics
 *	Ben Dooks <ben@simtec.co.uk>
 *
 * http://www.fluff.org/ben/smdk2416/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
*/

#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/interrupt.h>
#include <linux/list.h>
#include <linux/timer.h>
#include <linux/init.h>
#include <linux/serial_core.h>
#include <linux/platform_device.h>
#include <linux/io.h>

#include <linux/mtd/mtd.h>
#include <linux/mtd/nand.h>
#include <linux/mtd/nand_ecc.h>
#include <linux/mtd/partitions.h>

#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/irq.h>

#include <mach/hardware.h>
#include <asm/irq.h>
#include <asm/mach-types.h>

#include <plat/regs-serial.h>
#include <mach/regs-gpio.h>
#include <mach/regs-lcd.h>

#include <mach/idle.h>
#include <mach/fb.h>
#include <plat/iic.h>

#include <plat/s3c2410.h>
#include <plat/s3c2440.h>
#include <plat/clock.h>
#include <plat/devs.h>
#include <plat/cpu.h>
#include <plat/pm.h>
#include <plat/nand.h>

static struct s3c2410_uartcfg snp2100_uartcfgs[] __initdata = {
	[0] = {
		.hwport	     = 0,
		.flags	     = 0,
		.ucon	     = 0x3c5,
		.ulcon	     = 0x03,
		.ufcon	     = 0x51,
	},
	[1] = {
		.hwport	     = 1,
		.flags	     = 0,
		.ucon	     = 0x3c5,
		.ulcon	     = 0x03,
		.ufcon	     = 0x51,
	},
	[2] = {
		.hwport	     = 2,
		.flags	     = 0,
		.ucon	     = 0x3c5,
		.ulcon	     = 0x03,
		.ufcon	     = 0x51,
	}
};

struct mtd_partition snp2100_nand_part[] = {
        {
                .name           = "Bootloader",
                .offset         = 0,
                .size           = (256*SZ_1K),
                .mask_flags     = MTD_CAP_NANDFLASH,
        },
        {
                .name           = "Kernel",
                .offset         = (256*SZ_1K),    /* Block number is 0x10 */
                .size           = (2*SZ_1M) - (256*SZ_1K),
                .mask_flags     = MTD_CAP_NANDFLASH,
        },
        {
                .name           = "Root",
                .offset         = (2*SZ_1M),    /* Block number is 0x80 */
                .size           = (48*SZ_1M),
        },
        {
                .name           = "File System",
                .offset         = MTDPART_OFS_APPEND,
                .size           = MTDPART_SIZ_FULL,
        }
};

static struct s3c2410_nand_set snp2100_nand_sets[] = {
	[0] = {
		.name		= "NAND",
		.nr_chips	= 1,
		.nr_partitions	= ARRAY_SIZE(snp2100_nand_part),
		.partitions	= snp2100_nand_part,
	},
};

static struct s3c2410_platform_nand snp2100_nand_info = {
	.tacls		= 20,
	.twrph0		= 60, /* 50 */
	.twrph1		= 20,
	.nr_sets	= ARRAY_SIZE(snp2100_nand_sets),
	.sets		= snp2100_nand_sets,
};

static struct platform_device *snp2100_devices[] __initdata = {
	&s3c_device_nand,
#ifdef CONFIG_S3C_DEV_HSMMC
        &s3c_device_hsmmc0,
#endif
#ifdef CONFIG_S3C_DEV_HSMMC1
        &s3c_device_hsmmc1,
#endif
#if 0
	&s3c_device_wdt,
	&s3c_device_i2c0,
#endif
};

static void __init snp2100_map_io(void)
{
	s3c24xx_init_io(NULL, 0);
	s3c24xx_init_clocks(12000000);
	s3c24xx_init_uarts(snp2100_uartcfgs, ARRAY_SIZE(snp2100_uartcfgs));
}

static void __init snp2100_machine_init(void)
{
	s3c_device_nand.dev.platform_data = &snp2100_nand_info;
//	s3c_i2c0_set_platdata(NULL);
	platform_add_devices(snp2100_devices, ARRAY_SIZE(snp2100_devices));
	s3c2410_pm_init();
}

MACHINE_START(SNP2100, "SNP2100")
	.phys_io	= S3C2410_PA_UART,
	.io_pg_offst	= (((u32)S3C24XX_VA_UART) >> 18) & 0xfffc,
	.boot_params	= S3C2410_SDRAM_PA + 0x100,

	.init_irq	= s3c24xx_init_irq,
	.map_io		= snp2100_map_io,
	.init_machine	= snp2100_machine_init,
	.timer		= &s3c24xx_timer,
MACHINE_END
