/* linux/arch/arm/mach-s3c2443/mach-yup150.c
 *
 * Copyright (c) 2007 Simtec Electronics
 *	Ben Dooks <ben@simtec.co.uk>
 *
 * http://www.fluff.org/ben/smdk2416/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
*/

#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/interrupt.h>
#include <linux/list.h>
#include <linux/timer.h>
#include <linux/hrtimer.h>
#include <linux/init.h>
#include <linux/serial_core.h>
#include <linux/platform_device.h>
#include <linux/io.h>
#include <linux/fb.h>
#include <linux/i2c.h>
#include <linux/delay.h>
#include <linux/sysdev.h>
#include <linux/pda_power.h>

#include <linux/mtd/mtd.h>
#include <linux/mtd/nand.h>
#include <linux/mtd/nand_ecc.h>
#include <linux/mtd/partitions.h>
#include <linux/spi/spi.h>
#include <linux/spi/flash.h>
#include <linux/i2c/ami602.h>

#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/irq.h>

#include <mach/hardware.h>
#include <asm/irq.h>
#include <asm/mach-types.h>

#include <plat/regs-serial.h>
#include <mach/regs-gpio.h>
#include <mach/regs-gpiol.h>
#include <mach/regs-fb.h>
#include <mach/regs-irq.h>
#include <mach/regs-s3c2416-clock.h>
#include <mach/leds-gpio.h>

#include <mach/idle.h>
#include <plat/fb.h>
#include <plat/regs-iic.h>
#include <plat/iic.h>

#include <plat/s3c2410.h>
#include <plat/s3c2440.h>
#include <plat/clock.h>
#include <plat/devs.h>
#include <plat/cpu.h>
#include <plat/nand.h>
#include <plat/pm.h>
#include <mach/spi.h>

#ifdef CONFIG_TOUCHSCREEN_S3C2410
#include <mach/ts.h>
#include <../drivers/input/touchscreen/ts_filter_chain.h>
#ifdef CONFIG_TOUCHSCREEN_FILTER
#include <../drivers/input/touchscreen/ts_filter_linear.h>
#include <../drivers/input/touchscreen/ts_filter_mean.h>
#include <../drivers/input/touchscreen/ts_filter_median.h>
#include <../drivers/input/touchscreen/ts_filter_group.h>
#endif /* CONFIG_TOUCHSCREEN_FILTER */
#endif /* CONFIG_TOUCHSCREEN_S3C2410 */

int yup150_gps_enable_on_suspend = 0;
static int yup150_gps_power_on = 1;

#define SYSTEM_REV_YUP110	0
#define SYSTEM_REV_YUP110S	1
#define SYSTEM_REV_YUP110R	2
#define SYSTEM_REV_YUP150	3

/*
 * LED configuration
 */
static struct s3c24xx_led_platdata yup110_pdata_led = {
	.gpio	= S3C2443_GPH14,
	.flags	= S3C24XX_LEDF_TRISTATE,
	.name	= "test_led",
};

static struct platform_device yup110_test_led = {
	.name	= "s3c24xx_led",
	.id	= -1,
	.dev	= {
		.platform_data = & yup110_pdata_led,
	},
};


#ifdef CONFIG_TOUCHSCREEN_S3C2410
/*
 * Touchscreen configuration.
 */

#ifdef CONFIG_TOUCHSCREEN_FILTER
const static struct ts_filter_group_configuration yup150_ts_group = {
        .length = 12,
        .close_enough = 10,
        .threshold = 6,         /* At least half of the points in a group. */
        .attempts = 10,
};

const static struct ts_filter_median_configuration yup150_ts_median = {
        .extent = 20,
        .decimation_below = 3,
        .decimation_threshold = 8 * 3,
        .decimation_above = 4,
};

const static struct ts_filter_mean_configuration yup150_ts_mean = {
        .length = 4,
};

const static struct ts_filter_linear_configuration yup150_ts_linear = {
        .constants = {1, 0, 0, 0, 1, 0, 1},     /* Don't modify coords. */
        .coord0 = 0,
        .coord1 = 1,
};
#endif

const static struct ts_filter_chain_configuration yup150_filter_configuration[] =
{
#ifdef CONFIG_TOUCHSCREEN_FILTER
        {&ts_filter_group_api,          &yup150_ts_group.config},
        {&ts_filter_median_api,         &yup150_ts_median.config},
        {&ts_filter_mean_api,           &yup150_ts_mean.config},
        {&ts_filter_linear_api,         &yup150_ts_linear.config},
#endif
        {NULL, NULL},
};

const static struct s3c2410_ts_mach_info yup150_ts_cfg = {
        .delay = 10000,
        .presc = 0xff, /* slow as we can go */
        .filter_config = yup150_filter_configuration,
};
#endif /* CONFIG_TOUCHSCREEN_S3C2410 */


/*
 * UART configuration
 */
static struct s3c2410_uartcfg yup150_uartcfgs[] __initdata = {
	[0] = {	/* UART0 : DEBUG */
		.hwport	     = 0,
		.flags	     = 0,
		.ucon	     = 0x3c5,
		.ulcon	     = 0x03,
		.ufcon	     = 0x51,
	},
	[1] = {	/* UART1 : GPS */
		.hwport	     = 1,
		.flags	     = 0,
		.ucon	     = 0x3c5,
		.ulcon	     = 0x03,
		.ufcon	     = 0x51,
	},
	[2] = {	/* UART2 : RD */
		.hwport	     = 2,
		.flags	     = 0,
		.ucon	     = 0x3c5,
		.ulcon	     = 0x03,
		.ufcon	     = 0x51,
	},
	[3] = {	/* UART3 : console */
		.hwport	     = 3,
		.flags	     = 0,
		.ucon	     = 0x3c5,
		.ulcon	     = 0x03,
		.ufcon	     = 0x51,
	}
};

/*
 * nand configuration
 */
struct mtd_partition yup150_nand_part[] = {
        {
                .name           = "Bootloader",
                .offset         = 0x00000000,
                .size           = 0x00200000,
        },
        {
                .name           = "Front sysimg",
                .offset         = 0x00200000,
                .size           = 0x01800000,
        },
        {
                .name           = "User",
                .offset         = 0x03200000,
                .size           = MTDPART_SIZ_FULL,
        },
        {
                .name           = "Rear sysimg",
                .offset         = 0x01A00000,
                .size           = 0x01800000,
        },
        {
                .name           = "Rear sysimg1", /* for compatibility */
                .offset         = 0x01A00000,
                .size           = 0x01800000,
        },
};

static struct s3c2410_nand_set yup150_nand_sets[] = {
	[0] = {
		.name		= "NAND",
		.nr_chips	= 1,
		.nr_partitions	= ARRAY_SIZE(yup150_nand_part),
		.partitions	= yup150_nand_part,
	},
};

static struct s3c2410_platform_nand yup150_nand_info = {
	.tacls		= 20,
	.twrph0		= 30, /* 50 */
	.twrph1		= 20,
	.nr_sets	= ARRAY_SIZE(yup150_nand_sets),
	.sets		= yup150_nand_sets,
};

/*
 * frame buffer configuration
 */
static void yup150_fb_gpio_setup(void)
{
	int	i;
	for (i = 0; i <= 4; i++)
		s3c2410_gpio_cfgpin(S3C2410_GPIONO(S3C2410_GPIO_BANKC,i),
				    (0x02 << (i*2)));
	for (i = 8; i <= 15; i++)
		s3c2410_gpio_cfgpin(S3C2410_GPIONO(S3C2410_GPIO_BANKC,i),
				    (0x02 << (i*2)));
	for (i = 0; i <= 15; i++)
		s3c2410_gpio_cfgpin(S3C2410_GPIONO(S3C2410_GPIO_BANKD,i),
				    (0x02 << (i*2)));
}

static struct s3c_fb_pd_win yup150_fb_win0 = {
        /* this is to ensure we use win0 */
        .win_mode       = {
                .pixclock       = 116594,
                .left_margin    = 3,
                .right_margin   = 6,
                .upper_margin   = 8,
                .lower_margin   = 5,
                .hsync_len      = 41,
                .vsync_len      = 10,
                .xres           = 240,
                .yres           = 432,
		.refresh	= 65,
        },
	.virtual_x	= 240,
	.virtual_y	= 432 * 2,
        .max_bpp        = 32,
        .default_bpp    = 16,
};

static struct s3c_fb_platdata yup150_lcd_pdata __initdata = {
        .setup_gpio     = yup150_fb_gpio_setup,
        .win[0]         = &yup150_fb_win0,
        .vidcon0        = VIDCON0_VLCKEN,
        .vidcon1        = VIDCON1_INV_VCLK | VIDCON1_INV_HSYNC | VIDCON1_INV_VSYNC,
};

/*
 * SPI configuration
 */
static void yup150_spi_gpio_setup(struct s3c2410_spi_info *spi, int enable)
{
	/* LCD SPI CS */
	s3c2410_gpio_cfgpin(S3C2410_GPE6, S3C2410_GPIO_OUTPUT);
	s3c2410_gpio_setpin(S3C2410_GPE6, 1);
	s3c2410_gpio_pullup(S3C2410_GPE6, S3C24XX_GPIO_PULL_UP);

	/* Serial Flash SPI CS */
	s3c2410_gpio_cfgpin(S3C2410_GPE7, S3C2410_GPIO_OUTPUT);
	s3c2410_gpio_setpin(S3C2410_GPE7, 1);
	s3c2410_gpio_pullup(S3C2410_GPE7, S3C24XX_GPIO_PULL_UP);

        if (enable) {
                s3c2410_gpio_cfgpin(S3C2410_GPE13, S3C2410_GPE13_SPICLK0);
                s3c2410_gpio_cfgpin(S3C2410_GPE12, S3C2410_GPE12_SPIMOSI0);
                s3c2410_gpio_cfgpin(S3C2410_GPE11, S3C2410_GPE11_SPIMISO0);
                s3c2410_gpio_pullup(S3C2410_GPE11, S3C24XX_GPIO_PULL_UP);
                s3c2410_gpio_pullup(S3C2410_GPE13, S3C24XX_GPIO_PULL_UP);

		/* serial flash hold pin */
		s3c2410_gpio_cfgpin(S3C2410_GPE9, S3C2410_GPIO_OUTPUT);
		s3c2410_gpio_setpin(S3C2410_GPE9, 1);
		s3c2410_gpio_pullup(S3C2410_GPE9, S3C24XX_GPIO_PULL_UP);
        } else {
                s3c2410_gpio_cfgpin(S3C2410_GPE13, S3C2410_GPIO_INPUT);
                s3c2410_gpio_cfgpin(S3C2410_GPE11, S3C2410_GPIO_INPUT);
                s3c2410_gpio_pullup(S3C2410_GPE11, S3C24XX_GPIO_PULL_DOWN);
                s3c2410_gpio_pullup(S3C2410_GPE12, S3C24XX_GPIO_PULL_DOWN);
                s3c2410_gpio_pullup(S3C2410_GPE13, S3C24XX_GPIO_PULL_DOWN);

		/* serial flash hold pin */
		s3c2410_gpio_cfgpin(S3C2410_GPE9, S3C2410_GPIO_OUTPUT);
		s3c2410_gpio_setpin(S3C2410_GPE9, 0);
		s3c2410_gpio_pullup(S3C2410_GPE9, S3C24XX_GPIO_PULL_DOWN);
        }
}

static void yup150_spi_set_cs(struct s3c2410_spi_info *spi, int cs, int pol)
{
	if (cs == 0)
		s3c2410_gpio_setpin(S3C2410_GPE6, pol);
	else if (cs == 1)
		s3c2410_gpio_setpin(S3C2410_GPE7, pol);
}

static struct s3c2410_spi_info yup150_spi_pdata = {
	.num_cs		= 2,
	.bus_num	= 0,
	.gpio_setup	= yup150_spi_gpio_setup,
	.set_cs		= yup150_spi_set_cs,
};

/*
 * serial flash configuration
 */
static struct mtd_partition	yup150_spi_flash_parts[] = {
        {
                .name           = "lastpos",
                .offset         = 0x00000,
                .size           = 0x20000,
        },
        {
                .name           = "config",
                .offset         = 0x20000,
                .size           = 0x20000,
        },
};

static struct flash_platform_data       yup150_spi_flash_data = {
        .nr_parts = 2,
        .parts = yup150_spi_flash_parts,
};

static struct spi_board_info yup150_spi_device[] = {
        {
                .modalias = "btl302443",
                .bus_num = 0,
                .chip_select = 0,
                .max_speed_hz = 1 * 1000 * 1000,
        },
        {
                .modalias = "m25p80",
                .bus_num = 0,
                .chip_select = 1,
                .max_speed_hz = 20 * 1000 * 1000,
                .platform_data = &yup150_spi_flash_data,
        },
};

/*
 * I2C configuration
 */
static struct s3c2410_platform_i2c yup150_i2c_data0 __initdata = {
        .flags          = 0,
        .slave_addr     = 0x10,
        .bus_freq       = 100*1000,
#ifdef CONFIG_TOUCHSCREEN_S3C2410
	/* before PP */
        .max_freq       = 200 * 1000,
#else
	.max_freq	= 500 * 1000,
#endif
        .sda_delay      = S3C2410_IICLC_SDA_DELAY5 | S3C2410_IICLC_FILTER_ON,
};

/*
 * Backlight configuration
 */
static struct platform_device s3c_obd1601muv_device = {
	.name = "obd1601muv",
	.id = -1,
};

/*
 * AM1602 configuration
 */
static void yup50_ami602_init_platform_hw(void)
{
	/* busy */
	s3c2410_gpio_cfgpin(S3C2410_GPF1, S3C2410_GPF1_EINT1);
	s3c2410_gpio_pullup(S3C2410_GPF1, S3C24XX_GPIO_PULL_UP);

	/* trigger */
	s3c2410_gpio_cfgpin(S3C2410_GPG4, S3C2410_GPG4_OUTP);
	s3c2410_gpio_setpin(S3C2410_GPG4, 0);

}

static void yup50_ami602_exit_platform_hw(void)
{
	s3c2410_gpio_pullup(S3C2410_GPF1, S3C24XX_GPIO_PULL_NONE);
}

static void yup50_ami602_set_trg(int hl)
{
	s3c2410_gpio_setpin(S3C2410_GPG4, !hl);
}

static int yup150_ami602_get_busy(void)
{
	return ! s3c2410_gpio_getpin(S3C2410_GPF1);
}

static void yup150_ami602_udelay(unsigned int usec)
{
#if 0
	udelay(usec);
#else
	int	rv;
	ktime_t	expire;
	set_current_state(TASK_UNINTERRUPTIBLE);
	expire = ktime_set(0, usec * 1000);
	rv = schedule_hrtimeout(&expire, HRTIMER_MODE_REL);
	if (rv)
		printk(KERN_ERR "schedule_hrtimeout error(%d) at yup150_ami602_udelay\n", rv);
#endif
}

static struct ami602_platform_data	yup150_ami602_platform_data = {
	.irqflags	= IRQ_TYPE_EDGE_FALLING,
	.init_platform_hw = yup50_ami602_init_platform_hw,
	.exit_platform_hw = yup50_ami602_exit_platform_hw,
	.set_trg = yup50_ami602_set_trg,
	.get_busy = yup150_ami602_get_busy,
	.udelay = yup150_ami602_udelay,
};

static struct i2c_board_info ami602_i2c_dev __initdata = {
	.type = "ami602",
	.addr = 0x30,
	.platform_data = &yup150_ami602_platform_data,
	.irq = IRQ_EINT1,
};

static struct i2c_board_info ak4183_i2c_dev __initdata = {
	.type = "ak4183",
	.addr = 0x48,
	.irq = IRQ_EINT7,
};

/*
 * remocon configuration
 */
static struct platform_device yup110r_remocon_device = {
	.name = "yup110r-remocon",
	.id = -1,
};

/*
 * button configuration
 */
static struct platform_device yup150_btn_device = {
	.name = "yup150-button",
	.id = -1,
};

/*
 * Power supply configuration
 */
static struct platform_device yup150_power_device = {
	.name			= "yup150-power",
	.id			= -1,
};

static struct platform_device *yup150_devices[] __initdata = {
	&s3c_device_nand,
	&s3c_device_adc,
	&s3c_device_ts,
        &s3c_device_hsmmc1,
	&s3c_device_i2c0,
	&s3c_device_fb,
	&s3c_device_g2d,
	&s3c_device_hsspi,
	&s3c_obd1601muv_device,
};

static void __init yup150_map_io(void)
{
	s3c24xx_init_io(NULL, 0);
	s3c24xx_init_clocks(12000000);
	s3c24xx_init_uarts(yup150_uartcfgs, ARRAY_SIZE(yup150_uartcfgs));
}

#ifdef CONFIG_HSMMC_S3C
#include <plat/hsmmc.h>
extern struct s3c_hsmmc_cfg s3c_hsmmc1_platform;
#endif /* CONFIG_HSMMC_S3C */

static void yup150_codec_power_on_off(int on)
{
	s3c2410_gpio_cfgpin(S3C2410_GPB5, S3C2410_GPB5_OUTP);
	if (on) {
		s3c2410_gpio_setpin(S3C2410_GPB5, 1);
		s3c2410_gpio_pullup(S3C2410_GPB5, S3C24XX_GPIO_PULL_UP);
	} else {
		s3c2410_gpio_setpin(S3C2410_GPB5, 0);
		s3c2410_gpio_pullup(S3C2410_GPB5, S3C24XX_GPIO_PULL_DOWN);
	}
}

void yup150_gps_power_on_off(int on)
{
	/* gps enable/disable : GPG7 only */
	s3c2410_gpio_cfgpin(S3C2410_GPG7, S3C2410_GPG7_OUTP);
	if (on) {
		s3c2410_gpio_setpin(S3C2410_GPG7, 1);
		s3c2410_gpio_pullup(S3C2410_GPG7, S3C24XX_GPIO_PULL_UP);
	} else {
		s3c2410_gpio_setpin(S3C2410_GPG7, 0);
		s3c2410_gpio_pullup(S3C2410_GPG7, S3C24XX_GPIO_PULL_DOWN);
	}
}

static void yup150_nand_wp_on_off(int on)
{
	s3c2410_gpio_cfgpin(S3C2410_GPG5, S3C2410_GPG5_OUTP);
	if (on) {
		s3c2410_gpio_setpin(S3C2410_GPG5, 0);
		s3c2410_gpio_pullup(S3C2410_GPG5, S3C24XX_GPIO_PULL_DOWN);
	} else {
		s3c2410_gpio_setpin(S3C2410_GPG5, 1);
		s3c2410_gpio_pullup(S3C2410_GPG5, S3C24XX_GPIO_PULL_UP);
	}
}

static void yup150_sflash_wp_on_off(int on)
{
	s3c2410_gpio_cfgpin(S3C2410_GPE8, S3C2410_GPE8_OUTP);
	if (on) {
		s3c2410_gpio_setpin(S3C2410_GPE8, 0);
		s3c2410_gpio_pullup(S3C2410_GPE8, S3C24XX_GPIO_PULL_DOWN);
	} else {
		s3c2410_gpio_setpin(S3C2410_GPE8, 1);
		s3c2410_gpio_pullup(S3C2410_GPE8, S3C24XX_GPIO_PULL_UP);
	}
}

#ifdef CONFIG_PM
static int yup150_pm_suspend(struct sys_device *sd, pm_message_t state)
{
	printk(KERN_INFO "yup150_pm_suspend\n");

#if 0
	/* disable audio codec */
	yup150_codec_power_on_off(0);
#endif

#if 0
	if (! yup150_gps_enable_on_suspend) {
		yup150_gps_power_on_off(0);
	}
#endif

	/* Nand WP enable */
	yup150_nand_wp_on_off(1);

	/* Serial Flash WP enable */
	yup150_sflash_wp_on_off(1);

	return 0;
}

static int yup150_pm_resume(struct sys_device *sd)
{
	printk(KERN_INFO "yup150_pm_resume\n");

	/* enable audio codec */
	yup150_codec_power_on_off(1);

	/* gps enable */
	if (yup150_gps_power_on)
		yup150_gps_power_on_off(1);

	/* Nand WP disable */
	yup150_nand_wp_on_off(0);

	/* Serial Flash WP disable */
	yup150_sflash_wp_on_off(0);

	return 0;
}
#else /* CONFIG_PM */
#define yup150_pm_suspend	NULL
#define yup150_pm_resume	NULL
#endif /* CONFIG_PM */

static struct sysdev_class yup150_pm_sysclass = {
	.name		= "mach-yup150",
	.suspend	= yup150_pm_suspend,
	.resume		= yup150_pm_resume,
};

static struct sys_device yup150_pm_sysdev = {
	.cls		= & yup150_pm_sysclass,
};

void yup150_power_led_off(void)
{
	s3c2410_gpio_cfgpin(S3C2410_GPG6, S3C2410_GPIO_OUTPUT);
	s3c2410_gpio_setpin(S3C2410_GPG6, 0);
	s3c2410_gpio_pullup(S3C2410_GPG6, S3C24XX_GPIO_PULL_DOWN);
}

static void yup150_power_off(void)
{
	printk(KERN_INFO "powering system down...\n");

	/* gps off */
	yup150_gps_power_on_off(0);

	/* Backlight off */
	s3c2410_gpio_cfgpin(S3C2410_GPB0, S3C2410_GPIO_OUTPUT);
	s3c2410_gpio_setpin(S3C2410_GPB0, 0);
	s3c2410_gpio_pullup(S3C2410_GPB0, S3C24XX_GPIO_PULL_DOWN);
	msleep(2);

	/* Power LED off */
	yup150_power_led_off();

	/* SW_OFF */
	s3c2410_gpio_cfgpin(S3C2410_GPA13, S3C2410_GPIO_OUTPUT);
	s3c2410_gpio_setpin(S3C2410_GPA13, 0);

	while (1)
		;
}

static ssize_t gps_enable_on_suspend_show(struct kobject *kobj,
					  struct kobj_attribute *attr,
					  char *buf)
{
	return sprintf(buf, "%d\n", yup150_gps_enable_on_suspend);
}
static ssize_t gps_enable_on_suspend_store(struct kobject *kobj,
					   struct kobj_attribute *attr,
					   const char * buf, size_t n)
{
	int value;
	if (sscanf(buf, "%d", &value) != 1 ||
	    (value != 0 && value != 1))
	{
		printk(KERN_ERR "gps_enable_on_suspend_store: Invalid value\n");
		return -EINVAL;
	}
	yup150_gps_enable_on_suspend = value;
	return n;
}
static ssize_t gps_power_on_show(struct kobject *kobj,
				 struct kobj_attribute *attr,
				 char *buf)
{
	return sprintf(buf, "%d\n", yup150_gps_power_on);
}
static ssize_t gps_power_on_store(struct kobject *kobj,
				  struct kobj_attribute *attr,
				  const char * buf, size_t n)
{
	int value;
	if (sscanf(buf, "%d", &value) != 1 ||
	    (value != 0 && value != 1))
	{
		printk(KERN_ERR "gps_power_on_store: Invalid value\n");
		return -EINVAL;
	}
	yup150_gps_power_on = value;
	yup150_gps_power_on_off(yup150_gps_power_on);
	return n;
}
static struct kobj_attribute gps_enable_on_suspend_attr =
	__ATTR(gps_enable_on_suspend, 0644, gps_enable_on_suspend_show,
					    gps_enable_on_suspend_store);
static struct kobj_attribute gps_power_on_attr =
	__ATTR(gps_power_on, 0644, gps_power_on_show, gps_power_on_store);

static ssize_t rststat_show(struct kobject *kobj, struct kobj_attribute *attr,
			    char *buf)
{
	u32	rststat = __raw_readl(S3C2416_RSTSTAT);

	if (rststat & (1<<0))
		strcpy(buf, "nRESET\n");
	else if (rststat & (1<<2))
		strcpy(buf, "watchdog\n");
	else if (rststat & (1<<3))
		strcpy(buf, "sleep\n");
	else if (rststat & (1<<4))
		strcpy(buf, "deep-stop\n");
	else if (rststat & (1<<5))
		strcpy(buf, "swrst\n");
	else
		strcpy(buf, "unknown\n");

	return strlen(buf);
}
static ssize_t wkupstat_show(struct kobject *kobj, struct kobj_attribute *attr,
			     char *buf)
{
	u32	wkupstat = __raw_readl(S3C2416_WKUPSTAT);

	if (wkupstat & (1<<0))
		strcpy(buf, "eint\n");
	else if (wkupstat & (1<<1))
		strcpy(buf, "rtc\n");
	else if (wkupstat & (1<<4))
		strcpy(buf, "rtc_tick\n");
	else if (wkupstat & (1<<5))
		strcpy(buf, "batf\n");
	else
		strcpy(buf, "unknown\n");

	return strlen(buf);
}
static struct kobj_attribute rststat_attr =
	__ATTR(rststat, 0444, rststat_show, NULL);
static struct kobj_attribute wkupstat_attr =
	__ATTR(wkupstat, 0444, wkupstat_show, NULL);

static void __init yup150_init_pm(void)
{
	sysdev_class_register(&yup150_pm_sysclass);
        sysdev_register(&yup150_pm_sysdev);

	__raw_writel((1<<15), S3C2443_PWRCFG);
	s3c2410_pm_init();

	pm_power_off = yup150_power_off;

	if (system_rev == SYSTEM_REV_YUP150) {
		int	error;
		error = sysfs_create_file(power_kobj, &gps_enable_on_suspend_attr.attr);
		if (error)
			printk(KERN_ERR "sysfs_create_file failed: %d\n", error);
		error = sysfs_create_file(power_kobj, &gps_power_on_attr.attr);
		if (error)
			printk(KERN_ERR "sysfs_create_file failed: %d\n", error);
		error = sysfs_create_file(power_kobj, &rststat_attr.attr);
		if (error)
			printk(KERN_ERR "sysfs_create_file failed: %d\n", error);
		error = sysfs_create_file(power_kobj, &wkupstat_attr.attr);
		if (error)
			printk(KERN_ERR "sysfs_create_file failed: %d\n", error);
	}
}

static void __init yup150_get_system_rev(void)
{
	s3c2410_gpio_pullup(S3C2410_GPB2, S3C24XX_GPIO_PULL_UP);
	s3c2410_gpio_pullup(S3C2410_GPB3, S3C24XX_GPIO_PULL_UP);
	s3c2410_gpio_pullup(S3C2410_GPB4, S3C24XX_GPIO_PULL_UP);

	s3c2410_gpio_cfgpin(S3C2410_GPB2, S3C2410_GPB2_INP);
	s3c2410_gpio_cfgpin(S3C2410_GPB3, S3C2410_GPB3_INP);
	s3c2410_gpio_cfgpin(S3C2410_GPB4, S3C2410_GPB4_INP);

	system_rev = 0;

	if (s3c2410_gpio_getpin(S3C2410_GPB2))
		system_rev |= (1 << 0);
	if (s3c2410_gpio_getpin(S3C2410_GPB3))
		system_rev |= (1 << 1);
	if (s3c2410_gpio_getpin(S3C2410_GPB4))
		system_rev |= (1 << 2);

	s3c2410_gpio_pullup(S3C2410_GPB2, S3C24XX_GPIO_PULL_NONE);
	s3c2410_gpio_pullup(S3C2410_GPB3, S3C24XX_GPIO_PULL_NONE);
	s3c2410_gpio_pullup(S3C2410_GPB4, S3C24XX_GPIO_PULL_NONE);

	printk("System Revision : %d\n", system_rev);
}

static void __init yup150_machine_init(void)
{
	yup150_get_system_rev();

	/* enable audio codec */
	yup150_codec_power_on_off(1);

	/* gps enable */
	yup150_gps_power_on_off(1);

	/* Serial Flash WP disable */
	yup150_sflash_wp_on_off(0);

#if 0
	/* for debug */
	s3c2410_gpio_cfgpin(S3C2443_GPH14, S3C2443_GPH14_OUTP);
	s3c2410_gpio_setpin(S3C2443_GPH14, 0);
	s3c2410_gpio_pullup(S3C2443_GPH14, S3C24XX_GPIO_PULL_DOWN);
#endif

	s3c_fb_set_platdata(&yup150_lcd_pdata);

#ifdef CONFIG_TOUCHSCREEN_S3C2410
	set_s3c2410ts_info(&yup150_ts_cfg);
#elif defined(CONFIG_TOUCHSCREEN_AK4183)
	s3c2410_gpio_cfgpin(S3C2410_GPF7, S3C2410_GPF7_EINT7);
	s3c2410_gpio_pullup(S3C2410_GPF7, S3C24XX_GPIO_PULL_UP);
#endif

#ifdef CONFIG_HSMMC_S3C
	s3c2410_gpio_cfgpin(S3C2410_GPF4, S3C2410_GPF4_EINT4);
	s3c2410_gpio_pullup(S3C2410_GPF4, S3C24XX_GPIO_PULL_UP);
	s3c_hsmmc1_platform.irq_cd = IRQ_EINT4;
#endif /* CONFIG_HSMMC_S3C */
	s3c_device_nand.dev.platform_data = &yup150_nand_info;
	s3c_i2c0_set_platdata(&yup150_i2c_data0);
	s3c_device_hsspi.dev.platform_data = &yup150_spi_pdata;
	platform_add_devices(yup150_devices, ARRAY_SIZE(yup150_devices));
	spi_register_board_info(yup150_spi_device, ARRAY_SIZE(yup150_spi_device));
	if (system_rev == SYSTEM_REV_YUP150) {
		platform_device_register(&s3c_device_rtc),
		platform_device_register(&yup150_btn_device);
		platform_device_register(&yup150_power_device);
		i2c_register_board_info(0, &ami602_i2c_dev, 1);
		i2c_register_board_info(0, &ak4183_i2c_dev, 1);
	} else if (system_rev == SYSTEM_REV_YUP110R) {
		platform_device_register(&yup110r_remocon_device);
		platform_device_register(&yup110_test_led);
	} else {
		i2c_register_board_info(0, &ak4183_i2c_dev, 1);
		platform_device_register(&yup110_test_led);
	}

	yup150_init_pm();
}

MACHINE_START(YUP150, "YUP150")
	.phys_io	= S3C2410_PA_UART,
	.io_pg_offst	= (((u32)S3C24XX_VA_UART) >> 18) & 0xfffc,
	.boot_params	= S3C2410_SDRAM_PA + 0x100,

	.init_irq	= s3c24xx_init_irq,
	.map_io		= yup150_map_io,
	.init_machine	= yup150_machine_init,
	.timer		= &s3c24xx_timer,
MACHINE_END
