/* linux/arch/arm/mach-s3c2416/pm.c
 *
 * Copyright (c) 2006 Simtec Electronics
 *	Ben Dooks <ben@simtec.co.uk>
 *
 * http://armlinux.simtec.co.uk/.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
*/

#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/interrupt.h>
#include <linux/list.h>
#include <linux/timer.h>
#include <linux/init.h>
#include <linux/sysdev.h>
#include <linux/platform_device.h>
#include <linux/io.h>

#include <mach/hardware.h>
#include <asm/irq.h>
#include <asm/mach-types.h>

#include <mach/regs-gpio.h>
#include <mach/regs-gpioj.h>
#include <mach/regs-gpiol.h>
#include <mach/regs-dsc.h>
#include <mach/regs-power.h>
#include <mach/regs-s3c2443-clock.h>

#include <plat/cpu.h>
#include <plat/pm.h>

#include <plat/s3c2416.h>

static void s3c2416_pm_prepare(void)
{
	/* Write the magic value u-boot uses to check for resume into
	 * the INFORM0 register, and ensure INFORM1 is set to the
	 * correct address to resume from. */

	__raw_writel(0x2BED, S3C2412_INFORM0);
	__raw_writel(virt_to_phys(s3c2410_cpu_resume), S3C2412_INFORM1);
}

#ifdef CONFIG_MACH_YUP150
extern int yup150_gps_enable_on_suspend;
extern void yup150_power_led_off(void);
extern void yup150_gps_power_on_off(int on);
#endif
static void s3c2416_cpu_suspend(void)
{
	__raw_writel(0xfefe, S3C2443_RSTCON);
	__raw_writel(0xffff, S3C2443_OSCSET);

#ifdef CONFIG_MACH_YUP150
	if (machine_is_yup150()) {
		unsigned long misccr;
		misccr = __raw_readl(S3C24XX_MISCCR);
		misccr |= (1 << 12);
		__raw_writel(misccr, S3C24XX_MISCCR);

		/* UART0 RX (FIXME) */
		s3c2410_gpio_pullup(S3C2410_GPH1, S3C24XX_GPIO_PULL_NONE);

		/* Power LED off(FIXME) */
		yup150_power_led_off();

		if (! yup150_gps_enable_on_suspend) {
			/* gps disable : GPG7 only [because of power LED] */
			yup150_gps_power_on_off(0);
		}
	}
#endif

	__raw_writel(0x2BED, S3C2443_PWRMODE);
	while(1) {
		/* we should never get past here */

		panic("sleep resumed to originator?");
	}

}

static int s3c2416_pm_add(struct sys_device *sysdev)
{
	pm_cpu_prep = s3c2416_pm_prepare;
	pm_cpu_sleep = s3c2416_cpu_suspend;

	return 0;
}

static struct sleep_save s3c2416_sleep[] = {
	SAVE_ITEM(S3C2416_DSC0),
	SAVE_ITEM(S3C2416_DSC1),
	SAVE_ITEM(S3C2416_DSC2),
	SAVE_ITEM(S3C2416_DSC3),

	SAVE_ITEM(S3C2450_GPBSEL),
	SAVE_ITEM(S3C2450_GPESEL),

	SAVE_ITEM(S3C2440_GPJDAT),
        SAVE_ITEM(S3C2440_GPJCON),
        SAVE_ITEM(S3C2440_GPJUP),

	SAVE_ITEM(S3C2443_GPLDAT),
        SAVE_ITEM(S3C2443_GPLCON),
        SAVE_ITEM(S3C2443_GPLUDP),
};

static int s3c2416_pm_suspend(struct sys_device *dev, pm_message_t state)
{
	s3c2410_pm_do_save(s3c2416_sleep, ARRAY_SIZE(s3c2416_sleep));
	return 0;
}

static int s3c2416_pm_resume(struct sys_device *dev)
{
	__raw_writel(0x0, S3C2412_INFORM0);
	s3c2410_pm_do_restore(s3c2416_sleep, ARRAY_SIZE(s3c2416_sleep));
	return 0;
}

static struct sysdev_driver s3c2416_pm_driver = {
	.add		= s3c2416_pm_add,
	.suspend	= s3c2416_pm_suspend,
	.resume		= s3c2416_pm_resume,
};

static __init int s3c2416_pm_init(void)
{
	return sysdev_driver_register(&s3c2416_sysclass, &s3c2416_pm_driver);
}

arch_initcall(s3c2416_pm_init);
