/* linux/arch/arm/mach-s3c2416/s3c2416.c
 *
 * Copyright (c) 2007 Simtec Electronics
 *   Ben Dooks <ben@simtec.co.uk>
 *
 * Samsung S3C2416 Mobile CPU support
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
*/

#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/interrupt.h>
#include <linux/list.h>
#include <linux/timer.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/serial_core.h>
#include <linux/sysdev.h>
#include <linux/clk.h>
#include <linux/io.h>

#include <linux/mmc/card.h>
#include <linux/mmc/host.h>

#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/irq.h>

#include <mach/hardware.h>
#include <asm/irq.h>

#include <mach/regs-s3c2416-clock.h>
#include <mach/reset.h>
#include <mach/regs-gpio.h>
#include <mach/regs-gpioj.h>
#include <mach/regs-gpiol.h>

#include <plat/s3c2416.h>
#include <plat/devs.h>
#include <plat/cpu.h>
#if defined(CONFIG_HSMMC_S3C) || defined(CONFIG_HSMMC_S3C_MODULE)
#include <plat/hsmmc.h>
#else
#include <plat/sdhci.h>
#endif
#include <plat/regs-sdhci.h>

static struct map_desc s3c2416_iodesc[] __initdata = {
	IODESC_ENT(WATCHDOG),
	IODESC_ENT(CLKPWR),
	IODESC_ENT(TIMER),
};

struct sysdev_class s3c2416_sysclass = {
	.name = "s3c2416-core",
};

static struct sys_device s3c2416_sysdev = {
	.cls		= &s3c2416_sysclass,
};

static void s3c2416_hard_reset(void)
{
	__raw_writel(S3C2416_SWRST_RESET, S3C2443_SWRST);
}

#if defined(CONFIG_HSMMC_S3C) || defined(CONFIG_HSMMC_S3C_MODULE)

/*
 *  HS-MMC GPIO Set function for S3C2416 board
 */
void hsmmc_set_gpio (uint channel, uint width)
{
	switch (channel) {
        case 0:
		s3c2410_gpio_cfgpin(S3C2410_GPE5, S3C2410_GPE5_SDCLK);
		s3c2410_gpio_cfgpin(S3C2410_GPE6, S3C2410_GPE6_SDCMD);
		s3c2410_gpio_pullup(S3C2410_GPE5, S3C24XX_GPIO_PULL_UP);
		s3c2410_gpio_pullup(S3C2410_GPE6, S3C24XX_GPIO_PULL_UP);

		if (width == 1) {
			/* GPIO E : MMC DATA0[0] */
			s3c2410_gpio_cfgpin(S3C2410_GPE7, S3C2410_GPE7_SDDAT0);
		} else if (width == 4) {
			/* GPIO E : MMC DATA0[0:3] */
			s3c2410_gpio_cfgpin(S3C2410_GPE7, S3C2410_GPE7_SDDAT0);
			s3c2410_gpio_cfgpin(S3C2410_GPE8, S3C2410_GPE8_SDDAT1);
			s3c2410_gpio_cfgpin(S3C2410_GPE9, S3C2410_GPE9_SDDAT2);
			s3c2410_gpio_cfgpin(S3C2410_GPE10, S3C2410_GPE10_SDDAT3);
			s3c2410_gpio_pullup(S3C2410_GPE7, S3C24XX_GPIO_PULL_UP);
			s3c2410_gpio_pullup(S3C2410_GPE8, S3C24XX_GPIO_PULL_UP);
			s3c2410_gpio_pullup(S3C2410_GPE9, S3C24XX_GPIO_PULL_UP);
			s3c2410_gpio_pullup(S3C2410_GPE10, S3C24XX_GPIO_PULL_UP);
		}
                break;

        case 1:
		s3c2410_gpio_cfgpin(S3C2443_GPL8, S3C2450_GPL8_SD1CMD);
		s3c2410_gpio_cfgpin(S3C2443_GPL9, S3C2450_GPL9_SD1CLK);
		s3c2410_gpio_pullup(S3C2443_GPL8, S3C24XX_GPIO_PULL_UP);
		s3c2410_gpio_pullup(S3C2443_GPL9, S3C24XX_GPIO_PULL_UP);

                /* GPIO J : Chip detect, LED, Write Protect */
                s3c2410_gpio_cfgpin(S3C2443_GPJ13, S3C2450_GPJ13_SD1LED);
                s3c2410_gpio_cfgpin(S3C2443_GPJ14, S3C2450_GPJ14_nSD1CD);

                s3c2410_gpio_cfgpin(S3C2443_GPJ15, S3C2450_GPJ15_nSD1WP); /* write protect enable */
                s3c2410_gpio_setpin(S3C2443_GPJ15, 1);

		if (width == 1) {
			/* GPIO L : MMC DATA1[0] */
			s3c2410_gpio_cfgpin(S3C2443_GPL0, S3C2450_GPL0_SD1DAT0);
		} else if (width == 4) {
			/* GPIO L : MMC DATA1[0:3] */
			s3c2410_gpio_cfgpin(S3C2443_GPL0, S3C2450_GPL0_SD1DAT0);
			s3c2410_gpio_cfgpin(S3C2443_GPL1, S3C2450_GPL1_SD1DAT1);
			s3c2410_gpio_cfgpin(S3C2443_GPL2, S3C2450_GPL2_SD1DAT2);
			s3c2410_gpio_cfgpin(S3C2443_GPL3, S3C2450_GPL3_SD1DAT3);

			s3c2410_gpio_pullup(S3C2443_GPL0, S3C24XX_GPIO_PULL_UP);
			s3c2410_gpio_pullup(S3C2443_GPL1, S3C24XX_GPIO_PULL_UP);
			s3c2410_gpio_pullup(S3C2443_GPL2, S3C24XX_GPIO_PULL_UP);
			s3c2410_gpio_pullup(S3C2443_GPL3, S3C24XX_GPIO_PULL_UP);
		}
                break;

        default:
                break;
        }
}
EXPORT_SYMBOL_GPL(hsmmc_set_gpio);

#define HOST_CAPS (MMC_CAP_4_BIT_DATA | MMC_CAP_MMC_HIGHSPEED | MMC_CAP_SD_HIGHSPEED)

/* Channel 0 : added HS-MMC channel */
struct s3c_hsmmc_cfg s3c_hsmmc0_platform = {
        .hwport = 0,
        .enabled = 1,
        .host_caps = HOST_CAPS,
        .bus_width = 4,
        .highspeed = 0,

        /* ctrl for mmc */
        .fd_ctrl[MMC_TYPE_MMC] = {
                .ctrl2 = 0xC0000100,            /* ctrl2 for mmc */
                .ctrl3[SPEED_NORMAL] = 0,       /* ctrl3 for low speed */
                .ctrl3[SPEED_HIGH]   = 0,       /* ctrl3 for high speed */
                .ctrl4 = 0x3,
        },

        /* ctrl for sd */
        .fd_ctrl[MMC_TYPE_SD] = {
                .ctrl2 = 0xC0000100,            /* ctrl2 for sd */
                .ctrl3[SPEED_NORMAL] = 0,       /* ctrl3 for low speed */
                .ctrl3[SPEED_HIGH]   = 0,       /* ctrl3 for high speed */
                .ctrl4 = 0x3,
        },

        .clk_name[0] = "hsmmc",                 /* 1st clock source */
        .clk_name[1] = "esysclk",               /* 2nd clock source */
        .clk_name[2] = "hsmmc-ext",             /* 3rd clock source */
};


/* Channel 1 : default HS-MMC channel */
struct s3c_hsmmc_cfg s3c_hsmmc1_platform = {
        .hwport = 1,
        .enabled = 1,
        .host_caps = HOST_CAPS,
        .bus_width = 4,
        .highspeed = 0,

        /* ctrl for mmc */
        .fd_ctrl[MMC_TYPE_MMC] = {
                .ctrl2 = 0xC0000100,            /* ctrl2 for mmc */
                .ctrl3[SPEED_NORMAL] = 0,       /* ctrl3 for low speed */
                .ctrl3[SPEED_HIGH]   = 0,       /* ctrl3 for high speed */
                .ctrl4 = 0x3,
        },

        /* ctrl for sd */
        .fd_ctrl[MMC_TYPE_SD] = {
                .ctrl2 = 0xC0000100,            /* ctrl2 for sd */
                .ctrl3[SPEED_NORMAL] = 0,       /* ctrl3 for low speed */
                .ctrl3[SPEED_HIGH]   = 0,       /* ctrl3 for high speed */
                .ctrl4 = 0x3,
        },

        .clk_name[0] = "hsmmc",                 /* 1st clock source */
        .clk_name[1] = "esysclk",               /* 2nd clock source */
        .clk_name[2] = "hsmmc-ext",             /* 3rd clock source */
};

#else /* CONFIG_HSMMC_S3C */

#if defined(CONFIG_S3C_DEV_HSMMC) || defined(CONFIG_S3C_DEV_HSMMC1)
static char *s3c2416_hsmmc_clksrcs[4] = {
#if 0
        [0] = "hsmmc",
        [1] = "hsmmc",
        [2] = "esysclk",
//	[2] = "hsmmc-div",
//	[3] = "hsmmc-ext",
#else
        [2] = "esysclk",
#endif
};

static void s3c2416_setup_sdhci_cfg_card(struct platform_device *dev,
					 void __iomem *r,
					 struct mmc_ios *ios,
					 struct mmc_card *card)
{
#if 1
        u32 ctrl2, ctrl3;

        /* don't need to alter anything acording to card-type */

//        writel(S3C64XX_SDHCI_CONTROL4_DRIVE_9mA, r + S3C64XX_SDHCI_CONTROL4);

        ctrl2 = readl(r + S3C_SDHCI_CONTROL2);
        ctrl2 &= S3C_SDHCI_CTRL2_SELBASECLK_MASK;
        ctrl2 |= (S3C64XX_SDHCI_CTRL2_ENSTAASYNCCLR |
                  S3C64XX_SDHCI_CTRL2_ENCMDCNFMSK |
                  S3C_SDHCI_CTRL2_ENFBCLKRX |
                  S3C_SDHCI_CTRL2_DFCNT_NONE |
                  S3C_SDHCI_CTRL2_ENCLKOUTHOLD);

        if (ios->clock < 25 * 1000000)
                ctrl3 = (S3C_SDHCI_CTRL3_FCSEL3 |
                         S3C_SDHCI_CTRL3_FCSEL2 |
                         S3C_SDHCI_CTRL3_FCSEL1 |
                         S3C_SDHCI_CTRL3_FCSEL0);
        else
                ctrl3 = (S3C_SDHCI_CTRL3_FCSEL1 | S3C_SDHCI_CTRL3_FCSEL0);

        printk(KERN_INFO "%s: CTRL 2=%08x, 3=%08x\n", __func__, ctrl2, ctrl3);
        writel(ctrl2, r + S3C_SDHCI_CONTROL2);
//        writel(ctrl3, r + S3C_SDHCI_CONTROL3);
        writel(0, r + S3C_SDHCI_CONTROL3);
#else
	printk("%s at %d\n", __func__, dev->id);

	writel(0xC0000100, r + S3C_SDHCI_CONTROL2);
        writel(0, r + S3C_SDHCI_CONTROL3);
        writel(0x3 << 16, r + S3C64XX_SDHCI_CONTROL4);
#endif
}
#endif

#ifdef CONFIG_S3C_DEV_HSMMC
static void s3c2416_setup_sdhci0_cfg_gpio(struct platform_device *dev, int width)
{
	printk("%s: width(%d)\n", __func__, width);

	/* GPIO E : Command, Clock */
	s3c2410_gpio_cfgpin(S3C2410_GPE5, S3C2410_GPE5_SDCLK);
	s3c2410_gpio_cfgpin(S3C2410_GPE6, S3C2410_GPE6_SDCMD);

	if (width == 1) {
		/* GPIO E : MMC DATA0[0] */
		s3c2410_gpio_cfgpin(S3C2410_GPE7, S3C2410_GPE7_SDDAT0);
	} else if (width == 4) {
		/* GPIO E : MMC DATA0[0:3] */
		s3c2410_gpio_cfgpin(S3C2410_GPE7, S3C2410_GPE7_SDDAT0);
		s3c2410_gpio_cfgpin(S3C2410_GPE8, S3C2410_GPE8_SDDAT1);
		s3c2410_gpio_cfgpin(S3C2410_GPE9, S3C2410_GPE9_SDDAT2);
		s3c2410_gpio_cfgpin(S3C2410_GPE10, S3C2410_GPE10_SDDAT3);
	}
}
#endif

#ifdef CONFIG_S3C_DEV_HSMMC1
static void s3c2416_setup_sdhci1_cfg_gpio(struct platform_device *dev, int width)
{
	printk("%s: width(%d)\n", __func__, width);

        /* GPIO L : Command, Clock */
        s3c2410_gpio_cfgpin(S3C2443_GPL8, S3C2450_GPL8_SD1CMD);
        s3c2410_gpio_cfgpin(S3C2443_GPL9, S3C2450_GPL9_SD1CLK);

        if (width == 1) {
                /* GPIO L : MMC DATA1[0] */
                s3c2410_gpio_cfgpin(S3C2443_GPL0, S3C2450_GPL0_SD1DAT0);
        } else if (width == 4) {
                /* GPIO L : MMC DATA1[0:3] */
                s3c2410_gpio_cfgpin(S3C2443_GPL0, S3C2450_GPL0_SD1DAT0);
                s3c2410_gpio_cfgpin(S3C2443_GPL1, S3C2450_GPL1_SD1DAT1);
                s3c2410_gpio_cfgpin(S3C2443_GPL2, S3C2450_GPL2_SD1DAT2);
                s3c2410_gpio_cfgpin(S3C2443_GPL3, S3C2450_GPL3_SD1DAT3);
        }
}
#endif

#endif /* CONFIG_HSMMC_S3C */

int __init s3c2416_init(void)
{
	printk("S3C2416: Initialising architecture\n");

	s3c24xx_reset_hook = s3c2416_hard_reset;

#if 0
	s3c_device_nand.name = "s3c2412-nand";
#else
	s3c_device_nand.name = "s3c2416-nand";
#endif

#if !defined(CONFIG_HSMMC_S3C) && !defined(CONFIG_HSMMC_S3C_MODULE)
#ifdef CONFIG_S3C_DEV_HSMMC
	s3c_hsmmc0_def_platdata.clocks = s3c2416_hsmmc_clksrcs;
	s3c_hsmmc0_def_platdata.cfg_gpio = s3c2416_setup_sdhci0_cfg_gpio;
	s3c_hsmmc0_def_platdata.cfg_card = s3c2416_setup_sdhci_cfg_card;
#endif

#ifdef CONFIG_S3C_DEV_HSMMC1
	s3c_hsmmc1_def_platdata.clocks = s3c2416_hsmmc_clksrcs;
	s3c_hsmmc1_def_platdata.cfg_gpio = s3c2416_setup_sdhci1_cfg_gpio;
	s3c_hsmmc1_def_platdata.cfg_card = s3c2416_setup_sdhci_cfg_card;
#endif
#endif /* ! CONFIG_HSMMC_S3C */

	/* change WDT IRQ number */
	s3c_device_wdt.resource[1].start = IRQ_S3C2443_WDT;
	s3c_device_wdt.resource[1].end   = IRQ_S3C2443_WDT;

	return sysdev_register(&s3c2416_sysdev);
}

void __init s3c2416_init_uarts(struct s3c2410_uartcfg *cfg, int no)
{
	s3c24xx_init_uartdevs("s3c2440-uart", s3c2410_uart_resources, cfg, no);
}

/* s3c2416_map_io
 *
 * register the standard cpu IO areas, and any passed in from the
 * machine specific initialisation.
 */

void __init s3c2416_map_io(void)
{
	iotable_init(s3c2416_iodesc, ARRAY_SIZE(s3c2416_iodesc));

	s3c_device_i2c0.name  = "s3c2440-i2c";
}

/* need to register class before we actually register the device, and
 * we also need to ensure that it has been initialised before any of the
 * drivers even try to use it (even if not on an s3c2416 based system)
 * as a driver which may support both 2416 and 2440 may try and use it.
*/

static int __init s3c2416_core_init(void)
{
	return sysdev_class_register(&s3c2416_sysclass);
}

core_initcall(s3c2416_core_init);
