#include <linux/interrupt.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/miscdevice.h>
#include <linux/ioport.h>
#include <linux/fcntl.h>
#include <linux/init.h>
#include <linux/poll.h>
#include <linux/seq_file.h>
#include <linux/spinlock.h>
#include <linux/sysctl.h>
#include <linux/wait.h>
#include <linux/bcd.h>
#include <linux/delay.h>
#include <linux/platform_device.h>

#include <mach/hardware.h>
#include <mach/regs-gpio.h>

#define OBD1601MUV_MINOR	244

static int obd1601muv_open_cnt = 0;
static unsigned char obd1601muv_level = 40;

static void obd1601muv_set_level(unsigned char level)
{
	int i;

	s3c2410_gpio_cfgpin(S3C2410_GPB0, S3C2410_GPIO_OUTPUT);
	s3c2410_gpio_setpin(S3C2410_GPB0, 0);
	s3c2410_gpio_pullup(S3C2410_GPB0, S3C24XX_GPIO_PULL_UP);

	msleep(2);

	local_irq_disable();
	for (i = 0; i < level; i++) {
		if (i) {
			s3c2410_gpio_setpin(S3C2410_GPB0,0);
			udelay(2);
		}
 		s3c2410_gpio_setpin(S3C2410_GPB0,1);
		udelay(2);
	}
	local_irq_enable();
}

static ssize_t
obd1601muv_write(struct file *file, const char __user *buf, size_t count, loff_t *ppos)
{
	unsigned char level;

	if (copy_from_user(&level, buf, 1))
		return -EFAULT;

	if (level > 64)
		level = 64;
	else if (level && level <= 2)
		level += 64;

	obd1601muv_level = level;
	obd1601muv_set_level(level);

	return 1;
}

static int
obd1601muv_open(struct inode *inode, struct file *file)
{
	if (obd1601muv_open_cnt)
		return -EBUSY;
	obd1601muv_open_cnt++;
	return 0;
}

static int
obd1601muv_release(struct inode *inode, struct file *file)
{
	obd1601muv_open_cnt--;
	return 0;
}

static const struct file_operations obd1601muv_fops = {
	.owner		= THIS_MODULE,
	.write		= obd1601muv_write,
	.open		= obd1601muv_open,
	.release	= obd1601muv_release,
};

static struct miscdevice obd1601muv_dev = {
	OBD1601MUV_MINOR,
	"obd1601muv",
	&obd1601muv_fops
};

static int __init obd1601muv_probe(struct platform_device *pdev)
{
	int ret;

	ret = misc_register(&obd1601muv_dev);
	if (ret) {
		printk(KERN_ERR "obd1601muv: can't misc_register on minor=%d\n",
		       OBD1601MUV_MINOR);
		goto out;
	}
	ret = 0;
	printk(KERN_INFO "OBD1601MUV brightness control driver\n");
      out:
	return ret;
}

static int __exit obd1601muv_remove(struct platform_device *pdev)
{
	misc_deregister(&obd1601muv_dev);
	return 0;
}

#ifdef CONFIG_PM
static int obd1601muv_suspend(struct platform_device *pdev, pm_message_t state)
{
	obd1601muv_set_level(0);
	return 0;
}

static int obd1601muv_resume(struct platform_device *pdev)
{
	obd1601muv_set_level(obd1601muv_level);
	return 0;
}
#else
#define obd1601muv_suspend NULL
#define obd1601muv_resume  NULL
#endif

static struct platform_driver obd1601muv_driver = {
	.probe		= obd1601muv_probe,
	.remove		= __exit_p(obd1601muv_remove),
	.suspend	= obd1601muv_suspend,
	.resume		= obd1601muv_resume,
	.driver		= {
		.name	= "obd1601muv",
		.owner	= THIS_MODULE,
	},
};

static int __init obd1601muv_init(void)
{
	return platform_driver_register(& obd1601muv_driver);
}

static void __exit obd1601muv_exit(void)
{
	platform_driver_unregister(& obd1601muv_driver);
}

module_init(obd1601muv_init);
module_exit(obd1601muv_exit);

MODULE_LICENSE("GPL");
