struct s3c_g2d_pattern {
        u8      pattern[64];    /* RGB_565 format */
        u8      offsetX;
        u8      offsetY;
};

struct s3c_g2d_rop {
/* rop is combination of below 3 bit fields */
#define S3C_G2D_ROPV_SOURCE             (0xF0)
#define S3C_G2D_ROPV_DESTINATION        (0xCC)
#define S3C_G2D_ROPV_PATTERN            (0xAA)
        u8      rop;
#define S3C_G2D_ROP_OS_FG_COLOR         (0)
#define S3C_G2D_ROP_OS_PATTERN          (1)
        u8      rop_os;
};

struct s3c_g2d_alpha {
#define S3C_G2D_ROP_ALPHA_MODE_NO       (0)
#define S3C_G2D_ROP_ALPHA_MODE_PIXEL    (1)
#define S3C_G2D_ROP_ALPHA_MODE_GLOBAL   (2)
#define S3C_G2D_ROP_ALPHA_MODE_FADING   (1 << 2)
        u8      alpha_mode;
        u8      alpha;
        u8      fading;
};

struct s3c_g2d_stencil {
#define S3C_G2D_STENCIL_INVERSE (1 << 4)
#define S3C_G2D_STENCIL_ON_R    (1 << 3)
#define S3C_G2D_STENCIL_ON_G    (1 << 2)
#define S3C_G2D_STENCIL_ON_B    (1 << 1)
#define S3C_G2D_STENCIL_ON_A    (1 << 0)
#define S3C_G2D_STENCIL_OFF_ALL (0)
        u32     ctrl;
        u32     key_min;        /* ARGB_8888 */
        u32     key_max;        /* ARGB_8888 */
};


/* for clipping */
struct s3c_g2d_range {
        u16     x1, y1; /* 0 ~ 2039 */
        u16     x2, y2; /* 0 ~ 2039 */
};

#define S3C_G2D_ROTATE_0                (1 << 0)
#define S3C_G2D_ROTATE_90               (1 << 1)
#define S3C_G2D_ROTATE_180              (1 << 2)
#define S3C_G2D_ROTATE_270              (1 << 3)
#define S3C_G2D_ROTATE_X_FLIP           (1 << 4)
#define S3C_G2D_ROTATE_Y_FLIP           (1 << 5)
struct s3c_g2d_rotation {
        u16     mode;
        u16     ox, oy; /* 0 ~ 2039 */
};

#define S3C_G2D_COLOR_RGB_565           0
#define S3C_G2D_COLOR_RGBA_5551         1
#define S3C_G2D_COLOR_ARGB_1555         2
#define S3C_G2D_COLOR_RGBA_8888         3
#define S3C_G2D_COLOR_ARGB_8888         4
#define S3C_G2D_COLOR_XRGB_8888         5
#define S3C_G2D_COLOR_RGBX_8888         6

struct s3c_g2d_frame_buffer {
        u32     base_addr;      /* physical address */
        u16     w, h;           /* 1 ~ 2040 */
        u32     color_mode;
};

struct s3c_g2d_transparent {
#define S3C_G2D_BITBLT_TRANSPARENT      1
#define S3C_G2D_BITBLT_OPAQUE           0
        u32     mode;
        u32     bg_color;
        u32     bs_color;
};

struct s3c_g2d_bitblt_m2s {
#define S3C_G2D_BITBLT_NORMAL   1
#define S3C_G2D_BITBLT_STRETCH  2
        u32     mode;
        u32     incrx;
        u32     incry;
};

struct s3c_g2d_bitblt_rotate {
        struct s3c_g2d_alpha            alpha;
        u32                             rotation_mode;
	u32				ox, oy;
        struct s3c_g2d_frame_buffer     src_fb;
        struct s3c_g2d_frame_buffer     dst_fb;
	struct s3c_g2d_range		clip_range;
#if 0
        u32                             src_x, src_y;
        u32                             src_w, src_h;
        u32                             dst_x, dst_y;
#endif
        struct s3c_g2d_range            src_range;
        struct s3c_g2d_range            dst_range;
        struct s3c_g2d_transparent      transp;
};

#define S3C_G2D_MAGIC  	    0x8E
#define S3C_G2D_BITBLT      		_IO(S3C_G2D_MAGIC, 0x01)
#define S3C_G2D_SET_MEMORY  		_IO(S3C_G2D_MAGIC, 0x02)
#define S3C_G2D_BITBLT_HOST		_IO(S3C_G2D_MAGIC, 0x03)
